
from dataclasses import dataclass
from typing import Any, Callable

from .assertive import equals
from .exceptions import TestFunctionError
from .utils import *


@dataclass (init = False)
class Report:
    def __init__(self, summary: Callable[...,str], report: Callable[...,str], failure_stack: Callable[...,list[str]]):
        self.summary = summary
        self.report = report
        self.failure_stack = failure_stack
        

def my_test_ify() -> tuple[Callable[...,Any],Report]:
    _passed_tests_counter = 0
    _failed_tests_counter = 0
    _total_tests_counter = 0
    _failure_message: list[str] = []
    _success_message: list[str] = []
    _stacks: list[str] = []
    
    def summary()-> str:
        return get_summary(_total_tests_counter, _passed_tests_counter, _failed_tests_counter)
    
    def report()-> str:
        _report = ""
        for msg in _failure_message:
            _report = msg + _report
        for msg in _success_message:
            _report = msg + "\n" + _report
        return _report
    
    def failure_stack()-> list[str]:
        return _stacks[:] if len(_stacks) > 0 else [f"{GREEN}No failures{RESET}"]
    
    def run_test[T](
        expected:T, 
        result: T, 
        description:str="",
        test_function: Callable[..., Any] = equals,
        *args: Any,
        **kwargs: Any
    ) -> None:
        if not callable(test_function):
            raise TestFunctionError("The provided test function is not a valid function.")

        nonlocal _total_tests_counter
        nonlocal _passed_tests_counter
        nonlocal _failed_tests_counter
        nonlocal _success_message
        nonlocal _failure_message
        nonlocal _stacks
        
        _total_tests_counter += 1
        
        passed, msg, stack = test_function(expected, result, _total_tests_counter, description, *args, **kwargs)
        
        if stack is not None:
            _stacks.append(stack)
            
        if passed:
            _passed_tests_counter += 1
            _success_message.append(msg)
        else:
            _failed_tests_counter += 1
            _failure_message.append(msg)
    
    def core[T](
        expected:T, 
        result: T, 
        description:str="",
        test_function: Callable[..., Any] = equals,
        *args:list[Any], 
        **kwargs:dict[str,Any]
    ) -> None:

        run_test(expected, result, description, test_function, *args, **kwargs)
    
    return core, Report(summary, report, failure_stack)




from typing import Any

from .utils import GREEN, RED, RESET, YELLOW


class InvalidTypeError(Exception):
    """Exceção lançada quando um tipo incorreto é passado para uma função."""
    def __init__(self, expected_type: Any, received_type: Any, message: str="Tipo de dado incorreto"):
        self.expected_type = expected_type
        self.received_type = received_type
        super().__init__(f"{YELLOW}{message}: expected {GREEN}{expected_type}{YELLOW}, but received {RED}{received_type}{YELLOW}.{RESET}")


class TestFunctionError(Exception):
    """Exceção para problemas na função de teste fornecida."""
    def __init__(self, message: str="Erro com a função de teste fornecida"):
        super().__init__(message)


#__init__.py
from .assertive import (equals, greater_than, less_than, not_equals,
                        raises_exception)
from .exceptions import InvalidTypeError, TestFunctionError
from .my_test_ify import my_test_ify

__all__ = [
    "equals",
    "not_equals",
    "greater_than",
    "less_than",
    "raises_exception",
    "InvalidTypeError",
    "TestFunctionError",
    "my_test_ify"
]


from typing import Any, Callable, Type

from .exceptions import InvalidTypeError
from .utils import *


def equals[T](tested: T, expected: T, id_test: int, description: str="") -> tuple[bool, str, str|None]:
    if type(expected) != type(tested):
        raise InvalidTypeError(type(expected), type(tested), "Type Error: 'expected' and 'tested' values ​​must be the same type")
    
    if expected == tested:
        msg = f"{GREEN}✓ {description} -> Passed!{RESET}"
        return True, msg, None
    else:
        path, str_stack = error_location()
        msg = f"{RED}\n--------------------------------------------------"
        msg += f"\n{path}"
        msg += f"\n{RED}Test {id_test} failed: {description}"
        msg += f"\n{YELLOW}Expected: {expected}"
        msg += f"\n{RED   }Tested:   {tested}"
        msg += f"\n--------------------------------------------------{RESET}\n"
        return False, msg, str_stack
    
def not_equals[T](tested: T, expected: T, id_test: int, description: str="") -> tuple[bool, str, str|None]:
    if type(expected) != type(tested):
        raise InvalidTypeError(type(expected), type(tested), "Type Error: 'expected' and 'tested' values ​​must be the same type")
    
    if expected != tested:
        msg = f"{GREEN}✓ {description} -> Passed!{RESET}"
        return True, msg, None
    else:
        path, str_stack = error_location()
        msg = f"{RED}\n--------------------------------------------------"
        msg += f"\n{path}"
        msg += f"\n{RED}Test {id_test} failed: {description}"
        msg += f"\n{YELLOW}Expected value not equal to: {expected}"
        msg += f"\n{RED   }But the tested was equal to: {tested}"
        msg += f"\n--------------------------------------------------{RESET}\n"
        return False, msg, str_stack
    

def greater_than(tested: int|float, expected: int|float, id_test: int, description: str="") -> tuple[bool, str, str|None]:
    if not isinstance(tested, (int, float)):
        raise InvalidTypeError(type(expected), type(tested), f"Type Error: the 'tested' value must be int or float, but it is of{type(tested)}")
    if not isinstance(expected, (int, float)):
        raise InvalidTypeError(type(expected), type(tested), f"Type Error: the 'expected' value must be int or float, but it is of{type(expected)}")
    
    if type(expected) != type(tested):
        raise InvalidTypeError(type(expected), type(tested), "Type Error: 'expected' and 'tested' values ​​must be the same type")
    
    if tested > expected:
        msg = f"{GREEN}✓ {description} -> Passed!{RESET}"
        return True, msg, None
    else:
        path, str_stack = error_location()
        msg = f"{RED}\n--------------------------------------------------"
        msg += f"\n{path}"
        msg += f"\n{RED   }Test {id_test} failed: {description}"
        msg += f"\n{YELLOW}The tested value ({RED}{tested}{YELLOW}) must be greater than the expected value ({RED}{expected}{YELLOW})!"
        msg += f"\n{RED}--------------------------------------------------{RESET}\n"
        return False, msg, str_stack


def less_than(tested: int|float, expected: int|float, id_test: int, description: str="") -> tuple[bool, str, str|None]:
    if not isinstance(tested, (int, float)):
        raise InvalidTypeError(type(expected), type(tested), f"Type Error: the 'tested' value must be int or float, but it is of{type(tested)}")
    if not isinstance(expected, (int, float)):
        raise InvalidTypeError(type(expected), type(tested), f"Type Error: the 'expected' value must be int or float, but it is of{type(expected)}")
    
    if type(expected) != type(tested):
        raise InvalidTypeError(type(expected), type(tested), "Type Error: 'expected' and 'tested' values ​​must be the same type")
    
    if tested < expected:
        msg = f"{GREEN}✓ {description} -> Passed!{RESET}"
        return True, msg, None
    else:
        path, str_stack = error_location()
        msg = f"{RED}\n--------------------------------------------------"
        msg += f"\n{path}"
        msg += f"\n{RED   }Test {id_test} failed: {description}"
        msg += f"\n{YELLOW}The tested value ({RED}{tested}{YELLOW}) must be less than the expected value ({RED}{expected}{YELLOW})!"
        msg += f"\n{RED}--------------------------------------------------{RESET}\n"
        return False, msg, str_stack


def raises_exception(
    expected_exception:  Type[Exception], 
    func: Callable[...,Any], 
    id_test: int, 
    description: str="",
    *args:list[Any], 
    **kwargs:dict[str,Any]
) -> tuple[bool, str, str|None]:
    try:
        func(*args, **kwargs)
    except expected_exception as e:
        return True, f"{GREEN}✓ {description} -> Passed!{RESET}", None
    except Exception as e:
        path, str_stack = error_location()
        
        msg = f"{RED}\n--------------------------------------------------"
        msg += f"\n{path}"
        msg += f"\n{RED}Test {id_test} failed: {description}"
        msg += f"\n{YELLOW}Unexpected exception {RED}{e}{YELLOW} raised!"
        msg += f"\n{RED}--------------------------------------------------{RESET}\n"
        
        return False, msg, str_stack
    path, str_stack = error_location()
    msg = f"{RED}\n--------------------------------------------------"
    msg += f"\n{path}"
    msg += f"\n{RED}Test {id_test} failed: {description}"
    msg += f"\n{YELLOW}No exception raised, but expected {RED}{expected_exception.__name__}{YELLOW} raised!"
    msg += f"\n{RED}--------------------------------------------------{RESET}\n"
    return False, msg, str_stack



import inspect
import os
import re

WHITE = "\033[97m"
GREEN = "\033[92m"
RED = "\033[91m"
YELLOW = "\033[93m"
BLUE = "\033[94m"
RESET = "\033[0m"

def get_summary(_total_tests_counter: int, _passed_tests_counter: int, _failed_tests_counter: int) -> str:
    msg = f"{RESET}\n\n--------------------------------------------------"
    msg += f"\n{BLUE}SUMMARY OF TESTS"
    msg += f"\n{YELLOW} Total: {_total_tests_counter}"
    msg += f"\n{GREEN }Passed: {_passed_tests_counter}"
    msg += f"\n{RED   }Failed: {_failed_tests_counter}"
    msg += f"\n{RESET}--------------------------------------------------\n\n"
    return msg
    

def error_location() -> tuple[str, str]:
    stack = inspect.stack()
    frame = stack[-1]
    list_path: list[str] = re.split(r"[\\/]", frame.filename)
    code_context = frame.code_context[0].strip() if frame.code_context else ""
    file: str = list_path.pop()
    if os.name == 'nt':
        path: str = "\\".join(list_path) + "\\"
    elif os.name == 'posix':
        path: str = "/".join(path) + "/"


    path = f"{YELLOW}Method '{code_context}' was called {RED}on line {frame.lineno}{YELLOW} in file '{path}{RED}{file}'."
    return path, get_str_stack(stack)


def get_str_stack(stack: list[inspect.FrameInfo]) -> str:
    str_stack = f"{RED}\nSTACK:"

    for frame in stack:
            
        str_stack += f"\n{YELLOW}filename:     {WHITE}{frame.filename}"
        str_stack += f"\n{YELLOW}lineno:       {WHITE}{frame.lineno}"
        str_stack += f"\n{YELLOW}Code Context: {WHITE}{frame.code_context[0].strip() if frame.code_context else ''}"
        str_stack += f"\n{YELLOW}function:     {WHITE}{frame.function}"
        str_stack += "\n=============================================================================================="
    str_stack += f"\n{RED}END STACK!{RESET}\n"
    
    return str_stack
    
    
    



from setuptools import find_packages, setup

setup(
    name="my-test-ify",
    version="0.0.1",
    description="Micro framework de testes unitários minimalista",
    long_description=open("README.md", "r", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    author="Andre Luiz",
    author_email="andrebg28@gmail.com",
    url="https://github.com/andrebg28/my-test-ify.git",
    packages=find_packages(exclude=["tests*"]),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.12",
)